    Promise{T}()
    Promise()

Create a promise of type `T`; i.e., a memory location that holds a value of type `T` that
can be set once and retrieved asynchronously.

# Extended help

For an explanation on the concept, see:
[Futures and promises - Wikipedia](https://en.wikipedia.org/wiki/Futures_and_promises).

## Examples

```julia
julia> using ConcurrentUtils

julia> p = Promise{Int}();

julia> try_race_fetch(p)
Try.Err: NotSetError()

julia> put!(p, 123);

julia> fetch(p)
123

julia> try_race_put!(p, 456)
Try.Err: 123
```

## Memory ordering

An event that retrieves or waits for a value from a `promise` establishes a happened-before
edge from the event that have set the value to the `promise`.  Invocations of the API that
includes an event that sets a value to a `promise` include:

* `put!(promise, value)` that does not throw.
* `try_race_put!(promise, value)` that returns an `Ok` result.
* `try_race_put_with!(thunk, promise)` that calls `thunk` (and hence returns an `Ok`).
* `race_put_with!(thunk, promise)` that calls `thunk`.

(The invocation `thunk()` is sequenced-before the event that sets the value.)

Invocations of the API that includes an event that retrieves or waits for a value from a
`promise` include:

* `fetch(promise)`
* `wait(promise)`
* `try_race_fetch(promise)` that returns an `Ok` result.
* `try_race_put_with!(thunk, promise)` that does not call `thunk` (and hence returns an
  `Err`)
* `race_put_with!(thunk, promise)` that does not call `thunk`

## Supported operations

A `promise::Promise{T}` supports the following operations:

* `put!(promise, value)`: Set a `value`; throws an error if some value has already been set.
* [`try_race_put!(promise, value)`](@ref try_race_put!): Try to set a `value`.
* `fetch(promise)`: Obtain a value that the `promise` holds. Wait if necessary.
* `wait(promise)`: Wait for a value to be set.
* [`try_race_fetch(promise)`](@ref try_race_fetch): Try to obtain an existing value in the `promise`.
* [`try_race_put_with!(thunk, promise)`](@ref try_race_put_with!): Try to get a value or set
  a value generated by `thunk`.
* [`race_put_with!(thunk, promise)`](@ref): Similar to `try_race_put_with!` but  the
  returned value does not indicate if `thunk` is called or not.
